@@ -319,9 +319,12 @@ pub fn ewasm_fn_sig(item: TokenStream) -> TokenStream {
             .parse()
             .unwrap()
     } else {
-        format!("{}_SIG", item.to_string().to_ascii_uppercase())
-            .parse()
-            .unwrap()
+        format!(
+            "{}_SIG",
+            item.to_string().replace(" ", "").to_ascii_uppercase()
+        )
+        .parse()
+        .unwrap()
     }
 }
 
@@ -357,7 +360,7 @@ pub fn ewasm_fn_sig(item: TokenStream) -> TokenStream {
 /// ```
 #[proc_macro]
 pub fn ewasm_input_from(item: TokenStream) -> TokenStream {
-    let re = Regex::new(r"^(?P<contract>\w+),\s+(?P<name>\w+),?(?P<error_handler>.*)").unwrap();
+    let re = Regex::new(r"^(?P<contract>\w+),\s+(?P<name>[^,]+),?(?P<error_handler>.*)").unwrap();
     if let Some(cap) = re.captures(&item.to_string()) {
         let contract = cap.name("contract").unwrap().as_str();
         let fn_name = cap.name("name").unwrap().as_str();
@@ -409,6 +412,7 @@ pub fn ewasm_output_from(item: TokenStream) -> TokenStream {
 ///     description: String,
 /// }
 /// ```
+#[cfg(feature = "kv")]
 #[proc_macro_derive(Key)]
 pub fn derive_key(item: TokenStream) -> TokenStream {
     let re = Regex::new(r"struct (?P<name>\w+)").unwrap();
@@ -438,6 +442,7 @@ pub fn derive_key(item: TokenStream) -> TokenStream {
 ///     description: String,
 /// }
 /// ```
+#[cfg(feature = "kv")]
 #[proc_macro_derive(Value)]
 pub fn derive_value(item: TokenStream) -> TokenStream {
     let re = Regex::new(r"struct (?P<name>\w+)").unwrap();
@@ -457,6 +462,221 @@ pub fn derive_value(item: TokenStream) -> TokenStream {
     }
 }
 
+/// provides the handers for CRUD and the Protocol struct to communicate with these handlers.
+///
+/// ```compile_fail
+/// use sewup_derive::Table;
+/// #[derive(Table)]
+/// struct Person {
+///     trusted: bool,
+///     age: u8,
+/// }
+/// ```
+///
+/// The crud handlers are generated as `{struct_name}::get`, `{struct_name}::create`,
+/// `{struct_name}::update`, `{struct_name}::delete`, you can easily used these handlers as
+/// following example.
+///
+/// ```compile_fail
+/// #[ewasm_main]
+/// fn main() -> Result<()> {
+///     let mut contract = Contract::new()?;
+///
+///     match contract.get_function_selector()? {
+///         ewasm_fn_sig!(person::get) => ewasm_input_from!(contract, person::get)?,
+///         ewasm_fn_sig!(person::create) => ewasm_input_from!(contract, person::create)?,
+///         ewasm_fn_sig!(person::update) => ewasm_input_from!(contract, person::update)?,
+///         ewasm_fn_sig!(person::delete) => ewasm_input_from!(contract, person::delete)?,
+///         _ => return Err(RDBError::UnknownHandle.into()),
+///     }
+///
+///     Ok(())
+/// }
+/// ```
+///
+/// The protocol is the input and also the output format of these handlers, besides these handlers
+/// are easy to build by the `{struct_name}::protocol`, `{struct_name}::Protocol`, and use `set_id`
+/// for specify the record you want to modify.
+/// for examples.
+///
+/// ```compile_fail
+/// let handler_input = person::protocol(person);
+/// let mut default_person_input: person::Protocol = Person::default().into();
+/// default_input.set_id(2);
+/// ```
+///
+/// you can use `ewasm_output_from!` to get the exactly input/ouput binary of the protol, for
+/// example:
+/// ```
+/// let handler_input = person::protocol(person);
+/// ewasm_output_from!(handler_input)
+/// ```
+///
+/// Please note that the protocol default and the protocol for default instance may be different.
+/// This base on the implementation of the default trait of the structure.
+///
+/// ```compile_fail
+/// let default_input = person::Protocol::default();
+/// let default_person_input: person::Protocol = Person::default().into();
+/// assert!(default_input != default_person_input)
+/// ```
+#[cfg(feature = "rdb")]
+#[proc_macro_derive(Table)]
+pub fn derive_table(item: TokenStream) -> TokenStream {
+    let re = Regex::new(r"struct (?P<name>\w+)(?P<to_first_bracket>[^\{]*\{)(?P<fields>[^\}]*)}")
+        .unwrap();
+    if let Some(cap) = re.captures(&item.to_string()) {
+        let struct_name = cap.name("name").unwrap().as_str();
+        let field_part = cap.name("fields").unwrap().as_str();
+        let fields = field_part
+            .split(',')
+            .map(|p| p.split(':').nth(0).unwrap_or("").trim())
+            .filter(|s| !s.is_empty())
+            .collect::<Vec<_>>();
+        format!(
+            r#"
+            impl sewup::rdb::traits::Record for {} {{}}
+
+            #[derive(Default, Clone, sewup::Serialize, sewup::Deserialize)]
+            pub struct {}Wrapper {{
+                id: Option<usize>,
+                {}
+            }}
+
+            impl From<{}> for {}Wrapper {{
+                fn from(instance: {}) -> Self {{
+                    Self {{
+                        id: None,
+                        {}
+                    }}
+                }}
+            }}
+
+            impl From<{}Wrapper> for {} {{
+                fn from(wrapper: {}Wrapper) -> Self {{
+                    Self {{
+                        {}
+                    }}
+                }}
+            }}
+
+            mod {} {{
+                use sewup_derive::ewasm_fn_sig;
+
+                pub(crate) const GET_SIG: [u8; 4] = ewasm_fn_sig!({}::get());
+                pub(crate) const CREATE_SIG: [u8; 4] = ewasm_fn_sig!({}::create());
+                pub(crate) const UPDATE_SIG: [u8; 4] = ewasm_fn_sig!({}::update());
+                pub(crate) const DELETE_SIG: [u8; 4] = ewasm_fn_sig!({}::delete());
+            }}
+
+            #[cfg(target_arch = "wasm32")]
+            mod {} {{
+                use super::*;
+                pub fn get(instance: {}Wrapper) -> Result<()> {{
+                    let table = sewup::rdb::Db::load(None)?.table::<{}>()?;
+                    let raw_output = table.get_record(instance.id.unwrap_or_default())?;
+                    let mut output: {}Wrapper = raw_output.into();
+                    output.id = instance.id;
+                    sewup::utils::ewasm_return(ewasm_output_from!(output));
+                    Ok(())
+                }}
+                pub fn create(instance: {}Wrapper) -> Result<()> {{
+                    let mut table = sewup::rdb::Db::load(None)?.table::<{}>()?;
+                    let mut output = instance.clone();
+                    output.id = Some(table.add_record(instance.into())?);
+                    table.commit()?;
+                    sewup::utils::ewasm_return(ewasm_output_from!(output));
+                    Ok(())
+                }}
+                pub fn update(instance: {}Wrapper) -> Result<()> {{
+                    let mut table = sewup::rdb::Db::load(None)?.table::<{}>()?;
+                    let id = instance.id.unwrap_or_default();
+                    table.update_record(id, Some(instance.clone().into()))?;
+                    table.commit()?;
+                    sewup::utils::ewasm_return(ewasm_output_from!(instance));
+                    Ok(())
+                }}
+                pub fn delete(instance: {}Wrapper) -> Result<()> {{
+                    let mut table = sewup::rdb::Db::load(None)?.table::<{}>()?;
+                    let id = instance.id.unwrap_or_default();
+                    table.update_record(id, None)?;
+                    table.commit()?;
+                    sewup::utils::ewasm_return(ewasm_output_from!(instance));
+                    Ok(())
+                }}
+            }}
+
+            #[cfg(not(target_arch = "wasm32"))]
+            mod {} {{
+                use super::*;
+                #[inline]
+                pub fn protocol(instance: {}) -> {}Wrapper {{
+                    instance.into()
+                }}
+                pub type Protocol = {}Wrapper;
+                impl Protocol {{
+                    pub fn set_id(&mut self, id: usize) {{
+                        self.id = Some(id);
+                    }}
+                    pub fn is_empty(&self) -> bool {{
+                        self.trusted.is_none() && self.age.is_none()
+                    }}
+                }}
+            }}
+        "#,
+            //Record trait
+            struct_name,
+            // Wraper struct
+            struct_name,
+            field_part.replace(":", ":Option<").replace(",", ">,"),
+            // impl From for warper
+            struct_name,
+            struct_name,
+            struct_name,
+            fields
+                .iter()
+                .map(|f| format!("{}:Some(instance.{})", f, f))
+                .collect::<Vec<_>>()
+                .join(","),
+            // impl From for instance
+            struct_name,
+            struct_name,
+            struct_name,
+            fields
+                .iter()
+                .map(|f| format!(r#"{}:wrapper.{}.expect("{} field missing")"#, f, f, f))
+                .collect::<Vec<_>>()
+                .join(","),
+            // mod for signature
+            struct_name.to_ascii_uppercase(),
+            struct_name,
+            struct_name,
+            struct_name,
+            struct_name,
+            // mod for handlers
+            struct_name.to_ascii_lowercase(),
+            struct_name,
+            struct_name,
+            struct_name,
+            struct_name,
+            struct_name,
+            struct_name,
+            struct_name,
+            struct_name,
+            struct_name,
+            // mod for protocol
+            struct_name.to_ascii_lowercase(),
+            struct_name,
+            struct_name,
+            struct_name,
+        )
+        .parse()
+        .unwrap()
+    } else {
+        panic!("sewup-derive parsing struct fails: {}", item.to_string());
+    }
+}
+
 /// helps you setup the test mododule, and test cases in contract.
 ///
 /// ```compile_fail
@@ -577,7 +797,7 @@ pub fn ewasm_test(_attr: TokenStream, item: TokenStream) -> TokenStream {
 pub fn ewasm_assert_eq(item: TokenStream) -> TokenStream {
     let re = Regex::new(r"^(?P<fn_name>[^(]+?)\((?P<params>[^)]*?)\),(?P<equivalence>.*)").unwrap();
     if let Some(cap) = re.captures(&item.to_string().replace("\n", "")) {
-        let fn_name = cap.name("fn_name").unwrap().as_str();
+        let fn_name = cap.name("fn_name").unwrap().as_str().replace(" ", "");
         let params = cap.name("params").unwrap().as_str().replace(" ", "");
         let equivalence = cap.name("equivalence").unwrap().as_str();
         if params.is_empty() {
@@ -613,7 +833,7 @@ pub fn ewasm_assert_eq(item: TokenStream) -> TokenStream {
             .unwrap()
         }
     } else {
-        panic!("fail to parsing function in fn_select");
+        panic!("fail to parsing function in ewasm_assert_eq");
     }
 }
 
